@@ -336,6 +336,7 @@ class Snowflake(Dialect):
     class Parser(parser.Parser):
         IDENTIFY_PIVOT_STRINGS = True
         DEFAULT_SAMPLING_METHOD = "BERNOULLI"
+        COLON_IS_JSON_EXTRACT = True
 
         ID_VAR_TOKENS = {
             *parser.Parser.ID_VAR_TOKENS,
@@ -472,6 +473,14 @@ class Snowflake(Dialect):
             "TERSE USERS": _show_parser("USERS"),
         }
 
+        CONSTRAINT_PARSERS = {
+            **parser.Parser.CONSTRAINT_PARSERS,
+            "WITH": lambda self: self._parse_with_constraint(),
+            "MASKING": lambda self: self._parse_with_constraint(),
+            "PROJECTION": lambda self: self._parse_with_constraint(),
+            "TAG": lambda self: self._parse_with_constraint(),
+        }
+
         STAGED_FILE_SINGLE_TOKENS = {
             TokenType.DOT,
             TokenType.MOD,
@@ -482,44 +491,51 @@ class Snowflake(Dialect):
 
         SCHEMA_KINDS = {"OBJECTS", "TABLES", "VIEWS", "SEQUENCES", "UNIQUE KEYS", "IMPORTED KEYS"}
 
+        NON_TABLE_CREATABLES = {"STORAGE INTEGRATION", "TAG", "WAREHOUSE", "STREAMLIT"}
+
+        LAMBDAS = {
+            **parser.Parser.LAMBDAS,
+            TokenType.ARROW: lambda self, expressions: self.expression(
+                exp.Lambda,
+                this=self._replace_lambda(
+                    self._parse_conjunction(),
+                    expressions,
+                ),
+                expressions=[e.this if isinstance(e, exp.Cast) else e for e in expressions],
+            ),
+        }
+
+        def _parse_with_constraint(self) -> t.Optional[exp.Expression]:
+            if self._prev.token_type != TokenType.WITH:
+                self._retreat(self._index - 1)
+
+            if self._match_text_seq("MASKING", "POLICY"):
+                return self.expression(
+                    exp.MaskingPolicyColumnConstraint,
+                    this=self._parse_id_var(),
+                    expressions=self._match(TokenType.USING)
+                    and self._parse_wrapped_csv(self._parse_id_var),
+                )
+            if self._match_text_seq("PROJECTION", "POLICY"):
+                return self.expression(
+                    exp.ProjectionPolicyColumnConstraint, this=self._parse_id_var()
+                )
+            if self._match(TokenType.TAG):
+                return self.expression(
+                    exp.TagColumnConstraint,
+                    expressions=self._parse_wrapped_csv(self._parse_property),
+                )
+
+            return None
+
         def _parse_create(self) -> exp.Create | exp.Command:
             expression = super()._parse_create()
-            if isinstance(expression, exp.Create) and expression.kind == "TAG":
+            if isinstance(expression, exp.Create) and expression.kind in self.NON_TABLE_CREATABLES:
                 # Replace the Table node with the enclosed Identifier
                 expression.this.replace(expression.this.this)
 
             return expression
 
-        def _parse_column_ops(self, this: t.Optional[exp.Expression]) -> t.Optional[exp.Expression]:
-            this = super()._parse_column_ops(this)
-
-            casts = []
-            json_path = []
-
-            while self._match(TokenType.COLON):
-                path = super()._parse_column_ops(self._parse_field(any_token=True))
-
-                # The cast :: operator has a lower precedence than the extraction operator :, so
-                # we rearrange the AST appropriately to avoid casting the 2nd argument of GET_PATH
-                while isinstance(path, exp.Cast):
-                    casts.append(path.to)
-                    path = path.this
-
-                if path:
-                    json_path.append(path.sql(dialect="snowflake", copy=False))
-
-            if json_path:
-                this = self.expression(
-                    exp.JSONExtract,
-                    this=this,
-                    expression=self.dialect.to_json_path(exp.Literal.string(".".join(json_path))),
-                )
-
-                while casts:
-                    this = self.expression(exp.Cast, this=this, to=casts.pop())
-
-            return this
-
         # https://docs.snowflake.com/en/sql-reference/functions/date_part.html
         # https://docs.snowflake.com/en/sql-reference/functions-date-time.html#label-supported-date-time-parts
         def _parse_date_part(self: Snowflake.Parser) -> t.Optional[exp.Expression]:
@@ -716,6 +732,19 @@ class Snowflake(Dialect):
 
             return exp.var("".join(part.text for part in parts if part))
 
+        def _parse_lambda_arg(self) -> t.Optional[exp.Expression]:
+            this = super()._parse_lambda_arg()
+
+            if not this:
+                return this
+
+            typ = self._parse_types()
+
+            if typ:
+                return self.expression(exp.Cast, this=this, to=typ)
+
+            return this
+
     class Tokenizer(tokens.Tokenizer):
         STRING_ESCAPES = ["\\", "'"]
         HEX_STRINGS = [("x'", "'"), ("X'", "'")]
@@ -744,6 +773,8 @@ class Snowflake(Dialect):
             "TAG": TokenType.TAG,
             "TIMESTAMP_TZ": TokenType.TIMESTAMPTZ,
             "TOP": TokenType.TOP,
+            "WAREHOUSE": TokenType.WAREHOUSE,
+            "STREAMLIT": TokenType.STREAMLIT,
         }
 
         SINGLE_TOKENS = {
